import greenfoot.*;  // imports Actor, World, Greenfoot, GreenfootImage

import java.util.Random;
import java.awt.Color;

/**
 * Un monde où un act01 mange des feuilles.
 * 
 * @author Bernard Gisin
 * @version 0.1
 */

public class MyWorld extends World
{
   cAct01 aanAmp[][] = new cAct01[8][8];
    
   // compte le nombre de cliques
   public int nbCliques;
   
   // compte le nombre d'ampoules éteintes.
   public int nbAmpoulesEteinte;

   // nombre d'ampoules en X et en Y
   private int nbAmpoulesTotX = 4;
   private int nbAmpoulesTotY = 4;

   RectangleBas rectbas = null;
   Counter c1;
   
    /**
     * Crée le monde de 8 x 9 cellules.
     * La taille d'une cellule est de 60x60 pixels
     */
    public MyWorld() 
    {
        super(8, 9, 60);        
        setBackground("cell.jpg");
        
        populate();
    }

    /**
     * Place les ampoules, le fond du compteur et le compteur.
     */    
    public void populate()
    {
        int nX = 0; // coordonnées d'où les feuilles sont placées
        int nY = 0;

        // Place un quadrillage de 4 ampoules sur 4.
        for (nX=0; nX < nbAmpoulesTotX; nX++) {
            for (nY=0; nY < nbAmpoulesTotY; nY++) {
               aanAmp[nX][nY] = new cAct01();
               addObject(aanAmp[nX][nY], 1+nX, 1+nY);                
            }
        }

        // aucun clique sur une ampoule a été fait au départ.
        nbCliques = 0;

        nbAmpoulesEteinte = nbAmpoulesTotX * nbAmpoulesTotY;
        
        if (rectbas != null) {
            // une première initialisation a déjà été faite
            c1.fUpdate = true;
//            c1.setLocation(1, 8);
            return;
        }
        
        // Pour le fond du compteur
        rectbas = new RectangleBas();
        addObject(rectbas,3,8);
        
        GreenfootImage image = new GreenfootImage(540, 50);
        image.setColor(new Color(255, 255, 150));
        image.fillRect(0, 0, 540, 50);
        rectbas.setImage(image);
        
        // Pour le compteur de nombre de déplacement.
        c1 = new Counter();
        addObject(c1, 1, 8);
    }

    /**
     * Change l'état de l'ampoule nX, nY et de ses voisines.
     */    
    public void changeVoisins(int nX, int nY)
    {
       nbCliques++;
       
//       nbCliques = 100*nX + nY; // pour debugging
       
       changeUne(nX  , nY);
       changeUne(nX+1, nY);
       changeUne(nX-1, nY);
       changeUne(nX  , nY+1);
       changeUne(nX  , nY-1);        

//       nbCliques = nbAmpoulesEteinte; // pour debugging
       
       if (nbAmpoulesEteinte == 0) Termine();
    }

    /**
     * Change l'état de l'ampoule nX, nY, si elle existe.
     */    
    public void changeUne(int nX, int nY)
    {
       if ((nX < 0) || (nX >= nbAmpoulesTotX) || (nY < 0) || (nY >= nbAmpoulesTotY)) return;
       aanAmp[nX][nY].change();
    }
   
    /**
     * C'est terminé.
     */
    public void Termine()
    {
       Greenfoot.delay(10); // attend l'équivalent de 10 appels à "act()"
//       Greenfoot.playSound("BravoTermine.wav");  
       c1.makeImage("Terminé", "Nombre de cliques : ", nbCliques);
       Greenfoot.delay(50); // attend l'équivalent de xx appels à "act()"

       
      if (nbAmpoulesTotX > 5) {
          Greenfoot.stop();
          return;
        }

       nbAmpoulesTotX++;
       nbAmpoulesTotY++;
       populate();
    }

}